#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Icon=..\Morrowind 2.ico
#AutoIt3Wrapper_Res_Comment=Uses tes3cmd.exe and TESTool.exe to generate "MW Containers Animated exp.esp" to add animations by Arcimaestro Antares/scripts by qqqbbb to related containers having no script in your modlist
#AutoIt3Wrapper_Res_Description=Animated Containers Expansion Generator
#AutoIt3Wrapper_Res_Fileversion=1.0.0.2
#AutoIt3Wrapper_Res_Fileversion_AutoIncrement=p
#AutoIt3Wrapper_Res_LegalCopyright=by abot / MIT license
#AutoIt3Wrapper_AU3Check_Stop_OnWarning=y
#AutoIt3Wrapper_Run_Tidy=y
#AutoIt3Wrapper_Run_Au3Stripper=y
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
; aceg.au3
; by abot
; Generates animated containers expansion for all your loaded mods
; tes3cmd.exe and this file must be inside Data Files folder

#include <MsgBoxConstants.au3>
#include <FileConstants.au3>
#include <Misc.au3>
#include <Debug.au3>
;#include <String.au3>
;#include <Array.au3>
;#include <GUIConstantsEx.au3>
;#include <WindowsConstants.au3>
;#include <ListBoxConstants.au3>
;#include <GuiListBox.au3>

Const $progname = 'ACEG'

If _Singleton($progname, 1) = 0 Then
	MsgBox($MB_SYSTEMMODAL, 'Warning', 'An occurence of ' & $progname & ' is already running.')
	Exit
EndIf

Const $CRLF2 = @CRLF & @CRLF
Const $logName = StringLower($progname) & '_log.txt'
Const $logPath = @ScriptDir & '\' & $logName

Const $debug = True
If $debug Then
	FileDelete($logPath)
	_DebugSetup($progname & ' Log', False, 4, $logPath)
EndIf


If Not FileExists('..\Morrowind.ini') Then
	MsgBox($MB_SYSTEMMODAL, 'Error', 'Morrowind.ini not found.')
	Exit
EndIf

Const $tes3cmd = 'tes3cmd.exe'
Const $base = 'MW Containers Animated'
Const $base_esp = $base & '.esp'
Const $generated_name = $base & ' exp'
Const $generated_esp = $generated_name & '.esp'
Const $generated_baks = $generated_name & '~*.esp'

If Not FileExists($tes3cmd) Then
	$s = $tes3cmd & ' not found.' & $CRLF2 & 'It should be in the "' & @ScriptDir & '" folder.'
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf

Const $tes3cmdscript = 'aceg.pl'
If Not FileExists($tes3cmdscript) Then
	$s = $tes3cmdscript & ' not found.' & $CRLF2 & 'It should be in the "' & @ScriptDir & '" folder.'
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf

Const $modlistName = StringLower($progname) & '_mod_list.txt'

$s = $tes3cmd & ' active > ' & $modlistName
$s1 = ConsoleRunWait($s)

If FileGetSize($modlistName) < 12 Then
	$s = "Error while reading mod list from Morrowind.ini. Please verify your Morrowind.ini is not corrupted."
	MsgBox($MB_SYSTEMMODAL, "Error", $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf

$f = FileOpen($modlistName, $FO_READ)
If $f = -1 Then
	$s = 'Unable to open file ' & $modlistName
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf
$modlist = FileRead($f)
FileClose($f)
If @error Then
	$s = 'Error reading file' & $modlistName
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf

; strip [LOAD ORDER] first line and [N Active Plugins] last line
$modlist = StringRegExpReplace($modlist, "(?im)^\[.*\]\r\n", '')
$modlist = StringRegExpReplace($modlist, "(?im)^" & $base_esp & "\r\n", '') ; strip base mod name
$modlist = StringRegExpReplace($modlist, "(?im)^" & $generated_esp & "\r\n", '') ; strip generated mod name
$modlist = StringRegExpReplace($modlist, "(?im)^Merged_Objects\.esp\r\n", '') ; strip Merged_Objects.esp
$modlist = StringRegExpReplace($modlist, "(?im)^multipatch\.esp\r\n", '') ; strip multipatch.esp
$modlist = StringRegExpReplace($modlist, "(?im)^NOM 3\.0\.esp\r\n", '') ; strip NOM3.0 to avoid problems with water barrels

_DebugOut($CRLF2 & 'modlist:' & @CRLF & $modlist)

$f = FileOpen($modlistName, $FO_OVERWRITE)
If $f = -1 Then
	$s = 'Unable to open file ' & $modlistName
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf
$s1 = FileWrite($f, $modlist)
FileClose($f)
If Not $s1 Then
	$s = 'Error writing file' & $modlistName
	MsgBox($MB_SYSTEMMODAL, 'Error', $s)
	_DebugOut($CRLF2 & $s)
	Exit
EndIf

$generated_esp_path = @ScriptDir & '\' & $generated_esp
$generated_esp_time = FileGetTime($generated_esp_path, 0, 1) ; store timestamp
FileMove($generated_esp, $generated_esp & '.bak', 1) ; make a backup


$s = 'Animated Containers Expansion Generator' & @LF & @LF & 'by abot' & @LF & @LF & _
		'Uses tes3cmd.exe and TESTool.exe to generate animated containers expansion for all your loaded mods.' & @LF & @LF & _
		'Please wait...'
;SplashTextOn($progname, $s, 400, 200, 100, 100, 1, 'Tahoma')
SplashTextOn($progname, $s, 400, 200, 100, 100, 1, 'Arial')

$mods = StringRegExpReplace($modlist, "(?im)^(.*\.es[mp])\r\n", '"\1" ') ; linearize mod list
;_DebugOut( $CRLF2 & "mods:" & @CRLF & $mods)


; dump containers
;$s = $tes3cmd & ' dump --type CONT --no-match "script" --raw-with-header "' & $generated_esp & '" ' & $mods

; debug test only $mods = "BAR_SuranRiverviewHome_v1.0.esp"

$s = $tes3cmd & ' dump --type CONT --raw-with-header "' & $generated_esp & '" ' & $mods
$s1 = ConsoleRunWait($s)

$s = $tes3cmd & ' modify --program-file "' & $tes3cmdscript & '" "' & $generated_esp & '" >> "' & $logName & '"'
$s1 = ConsoleRunWait($s)

; strip any container without script
$s = $tes3cmd & ' delete --type CONT --no-match "script" "' & $generated_esp & '"'
$s1 = ConsoleRunWait($s)

; strip any container without ac_
$s = $tes3cmd & ' delete --type CONT --no-match "ac_" "' & $generated_esp & '"'
$s1 = ConsoleRunWait($s)

$s = $tes3cmd & ' header --author "tes3cmd & abot" --description "' & _
		'Adds animations by Arcimaestro Antares/scripts by qqqbbb to related containers having no script in your modlist.\n\n' & _
		'Requires:\nMW Containers Animated.esp\n\n' & _
		'Load as last as possible, right before merged objects and leveled lists." "' & $generated_esp & '"'
$s1 = ConsoleRunWait($s)

;$s = $tes3cmd & ' dump "' & $generated_esp & '" >> "' & $logName & '"'
;$s1 = ConsoleRunWait($s)

RunTESTool2CleanDuplicatedRecords()

$s = $tes3cmd & ' dump "' & $generated_esp & '" >> "' & $logName & '"'
$s1 = ConsoleRunWait($s)

FileDelete($generated_baks)

If $generated_esp_time Then
	FileSetTime($generated_esp_path, $generated_esp_time)
EndIf

If $debug Then
	ShellExecute($logPath) ; open log with standard program
EndIf

SplashOff()



Func ConsoleRunWait($command)
	_DebugOut($CRLF2 & $command)
	Local $result = RunWait(@ComSpec & ' /c ' & $command, '', @SW_HIDE)
	Return $result
EndFunc   ;==>ConsoleRunWait

Func RunTESTool2CleanDuplicatedRecords()

	Const $TESTool_logName = 'TESTool.log'
	Const $TESTool_iniName = 'TESTool.ini'

	If Not FileExists('..\TESTool.exe') Then
		$s = "Can't clean generated mod from duplicates as TESTool.exe is not found. " & _
				"TESTool.exe should be in the same folder as Morrowind.exe."
		MsgBox($MB_SYSTEMMODAL, 'Warning', $s)
		_DebugOut($CRLF2 & $s)
		Return 1
	EndIf

	FileChangeDir(@ScriptDir & '/..')
	Sleep(500) ; IMPORTANT!!!
	$TESTool_logPath = @WorkingDir & '\' & $TESTool_logName
	$TESTool_iniPath = @WorkingDir & '\' & $TESTool_iniName
	$KeepFileName = IniRead($TESTool_iniPath, 'Cleaning Settings', 'KeepFileName', '1')
	$KeepFileDate = IniRead($TESTool_iniPath, 'Cleaning Settings', 'KeepFileDate', '1')
	
	Run('TESTool.exe', '')

	$s = WinWaitActive("Morrowind Location", "Morrowind.ini has been found in the current folder", 5)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		ControlClick($s, '', 6)
		_DebugOut($CRLF2 & "ControlClick($s, '', 6)")
	Else
		Return 2
	EndIf

	$s = WinWaitActive('TES Plugin Tool', 'Execution Log', 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		ControlClick($s, '', 1008) ; Options
		_DebugOut($CRLF2 & "ControlClick($s, '', 1008) ; Options")
	Else
		Return 3
	EndIf

	$s = WinWaitActive('TES Plugin Tool', "Don't change plugin filenames", 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		ControlCommand($s, '', 1007, 'Check') ; Don't change plugin filenames ON
		_DebugOut($CRLF2 & "ControlCommand($s, '', 1007, 'Check') ; Don't change plugin filenames ON")
		Sleep(500)
		ControlCommand($s, '', 1006, 'Check') ; Retain file time, when cleaning ON
		_DebugOut($CRLF2 & "ControlCommand($s, '', 1006, 'Check') ; Retain file time, when cleaning ON")
		Sleep(500)
		ControlClick($s, '', 1009) ; Done
		_DebugOut($CRLF2 & 'ControlClick(' & $s & ', '', 1009) ; Done')
	Else
		Return 4
	EndIf

	$s = WinWaitActive('TES Plugin Tool', 'Execution Log', 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		ControlCommand($s, '', 1004, 'SelectString', 'Clean ESP/ESM files')
		Sleep(500)
		ControlClick($s, '', 1001) ; Execute
		_DebugOut($CRLF2 & "ControlCommand($s, '', 1004, 'SelectString', 'Clean ESP/ESM files')")
		_DebugOut($CRLF2 & "ControlClick($s, '', 1001) ; Execute")
	Else
		Return 5
	EndIf

	$s = WinWaitNotActive('TES Plugin Tool', 'Execution Log', 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		Send($generated_esp_path & '{ENTER}')
		_DebugOut($CRLF2 & "Send(" & $generated_esp_path & "'{ENTER}')")
	Else
		Return 6
	EndIf

	$s = WinWaitActive('TESTool', 'contains duplicate objects', 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		ControlClick($s, '', 6) ; Yes
		_DebugOut($CRLF2 & "ControlClick($s, '', 6) ; Yes")
	Else
		; Return 7
	EndIf

	$s = WinWaitActive('TES Plugin Tool', 'Execution Log', 1)
	If $s Then
		WinSetOnTop($s, "", 1)
		Sleep(500)
		Send('!{F4}')
		_DebugOut($CRLF2 & "Send('!{F4}')")
	Else
		Return 8
	EndIf

	FileChangeDir(@ScriptDir) ; back to "Data Files" folder
	Sleep(500) ; IMPORTANT!!!
	$testool_log = ''
	If $debug Then
		$f = FileOpen($TESTool_logPath, $FO_READ)
		If $f = -1 Then
			$s = 'Unable to open file ' & $TESTool_logPath
			MsgBox($MB_SYSTEMMODAL, 'Error', $s)
			_DebugOut($CRLF2 & $s)
			Return 9
		EndIf
		$testool_log = FileRead($f)
		FileClose($f)
		If @error Then
			$s = 'Error reading file' & $TESTool_logPath
			MsgBox($MB_SYSTEMMODAL, 'Error', $s)
			_DebugOut($CRLF2 & $s)
			Return 10
		EndIf
	EndIf

	If $debug Then
		If $testool_log Then
			; append TESTool log to log
			$f = FileOpen($logPath, $FO_APPEND)
			If $f = -1 Then
				$s = 'Unable to open file ' & $logPath
				MsgBox($MB_SYSTEMMODAL, 'Error', $s)
				_DebugOut($CRLF2 & $s)
				Return 11
			EndIf
			$s1 = FileWrite($f, $CRLF2 & $testool_log)
			FileClose($f)
			If Not $s1 Then
				$s = 'Error writing file' & $logPath
				MsgBox($MB_SYSTEMMODAL, 'Error', $s)
				_DebugOut($CRLF2 & $s)
				Return 12
			EndIf
		EndIf
	EndIf

	$s = 'Cleaning Settings'
	If $KeepFileName = '0' Then
		IniWrite($TESTool_iniPath, $s, 'KeepFileName', $KeepFileName)
	EndIf
	If $KeepFileDate = '0' Then
		IniWrite($TESTool_iniPath, $s, 'KeepFileDate', $KeepFileDate)
	EndIf

EndFunc   ;==>RunTESTool2CleanDuplicatedRecords
